# -*- coding: utf-8 -*-

from gisdo.index import BaseIndex
from gisdo.algorithm import (
    ShortStayQueueAlgorithm, PrivilegeQueueAlgorithm,
    ActualCallAlgorithm, DisabledHealthNeedAlgorithm,
    CorrectHealthNeedAlgorithm, HealthNeedAlgorithm,
    DeferredQueueAlgorithm, CommonQueueAlgorithm,
    CurrentDateDeferredQueueAlgorithm, CurrentCalendarActualCallAlgorithm,
    CurrentDateActualCallAlgorithm, DeferredInFutureAlgorithm,
    QueueInCurrentCalendarAlgorithm, DeferredQueueInNextYear,
    QueueInNextYear, QueueInThisYear, SoftDeferredQueueAlgorithm)
from gisdo.constants import GOVERMENT


class QueueIndex(BaseIndex):

    ACTUAL_CALL = 1
    PRIVILEGE = 2
    SHORT_STAY = 3
    DISABLED = 4
    CORRECT = 5
    HEALTH = 6
    DEFERRED = 7
    COMMON_QUEUE = 8
    CURRENT_DATE_DEFERRED = 9
    CURRENT_CALENDAR_ACTUAL_CALL = 10
    CURRENT_DATE_ACTUAL_CALL = 11
    DEFERRED_IN_FUTURE = 12
    QUEUE_IN_CURRENT_CALENDAR_YEAR = 13
    QUEUE_IN_NEXT_YEAR = 14
    DEFERRED_QUEUE_IN_NEXT_YEAR = 15
    QUEUE_IN_THIS_YEAR = 16
    SOFT_DEFERRED = 17

    ALGORITHM_MAP = {
        SHORT_STAY: ShortStayQueueAlgorithm,
        PRIVILEGE: PrivilegeQueueAlgorithm,
        ACTUAL_CALL: ActualCallAlgorithm,
        DISABLED: DisabledHealthNeedAlgorithm,
        CORRECT: CorrectHealthNeedAlgorithm,
        HEALTH: HealthNeedAlgorithm,
        DEFERRED: DeferredQueueAlgorithm,
        COMMON_QUEUE: CommonQueueAlgorithm,
        CURRENT_DATE_DEFERRED: CurrentDateDeferredQueueAlgorithm,
        CURRENT_CALENDAR_ACTUAL_CALL: CurrentCalendarActualCallAlgorithm,
        CURRENT_DATE_ACTUAL_CALL: CurrentDateActualCallAlgorithm,
        DEFERRED_IN_FUTURE: DeferredInFutureAlgorithm,
        QUEUE_IN_CURRENT_CALENDAR_YEAR: QueueInCurrentCalendarAlgorithm,
        QUEUE_IN_NEXT_YEAR: QueueInNextYear,
        DEFERRED_QUEUE_IN_NEXT_YEAR: DeferredQueueInNextYear,
        QUEUE_IN_THIS_YEAR: QueueInThisYear,
        SOFT_DEFERRED: SoftDeferredQueueAlgorithm
    }

    def __init__(self, unit, queue_cache):

        super(QueueIndex, self).__init__(unit, session=None)
        self.dou_type = self.unit.dou_type
        self._cache = queue_cache

    def get_count(
            self, dou_type=GOVERMENT,
            index_type=None, **kwargs):

        if ((not self.dou_type and dou_type != GOVERMENT) or
                (self.dou_type and self.dou_type.code != dou_type)):
            return 0

        qs = QueueIndex.get_algorithm(
            index_type)(
                unit=self.unit,
                cache=self._cache).get_result_set(**kwargs)

        return len(qs)
